/*
 * program: psfilt
 * file: fonts.c
 *
 * Copyright  1992 1993 Robert Joop
 *
 * handling of external fonts and fontfamilies
 * a fontfamily usually consists of a normal, italic, bold and bolditalic font
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: fonts.c,v $
 * Revision 1.4  1994/08/20  11:20:24  rj
 * #if 0 ... #endif code removed.
 *
 * Revision 1.3  1994/07/09  16:43:40  rj
 * a lot of const's removed and added
 *
 * Revision 1.2  1994/01/09  23:45:43  rj
 * PPD parser fr version 4 PPD files total umgeschrieben.
 * partieller support fr perl.
 *
 * Revision 1.1.1.1  1993/12/31  20:56:38  rj
 * erster cvs import.
 *
 */

static const char RCSId[] = "$Id: fonts.c,v 1.4 1994/08/20 11:20:24 rj Exp $";

#include "psfilt.h"
#include "fonts.h"
#include "cache.h"
#include "mem.h"
#include "str.h"
#include "verbose.h"
#include "error.h"

/*\[sep]--------------------------------------------------------------------------------------------------------------------------*/
t_list	*fontdirs;

/*\[sep]--------------------------------------------------------------------------------------------------------------------------*/
int fontpcmp (const t_font *l, const t_font *r)
{
assert (l->fontname);
assert (r->fontname);
  return strcmp (l->fontname, r->fontname);
}

int fontppcmp (const t_font **l, const t_font **r)
{
  return strcmp ((*l)->fontname, (*r)->fontname);
}

void freefont (t_font *font)
{
  /* font->fontname points into one of font->afms fontnames */
  font->afms = avldestroy (font->afms);
  font->fonts = avldestroy (font->fonts);
}

/*\[sep]--------------------------------------------------------------------------------------------------------------------------*/
void clearfontfile (t_fontfile *fontfile)
{
  clearfileinfo (&fontfile->file);
  fontfile->fontname = NULL;
  fontfile->version = VERSION_unknown;
}

int fontfilepcmpfilename (const t_fontfile *l, const t_fontfile *r)
{
  return fileinfopcmpfilename (&l->file, &r->file);
}

int fontfilepcmpversion (const t_fontfile *l, const t_fontfile *r)
{
  return versioncmp (l->version, r->version);
}

int fontfilepcmp (const t_fontfile *l, const t_fontfile *r)
{
  int rc;

  if (rc = str0cmp (l->fontname, r->fontname))
    return rc;
  if (rc = versioncmp (l->version, r->version))
    return rc;
  if (rc = fileinfopcmpfilename (&l->file, &r->file))
    return rc;
  ierror ("fontfilecmp");
  abort();/* standard C doesn't recognize volatile for non-returning functions. but gcc knows that abort() and exit() don't return... */
}

int printfontfile (const t_fontfile *fontfile, bool verbose)
{
  t_buf	fontversion;

  fprintf (stderr, "FontName: %s ", fontfile->fontname);
  fprintf (stderr, "Version: %s\n", strversion (fontversion, fontfile->version));
  return feof (stderr) ? fault : ok;
}

void freefontfile (t_fontfile *fontfile)
{
  freefileinfo (&fontfile->file);
}
