/*
 * program: psfilt
 * file: iso8859.c
 *
 * Copyright  1992 1993 Robert Joop
 *
 *	convert ISO-8859-1-codes (ISO Latin 1) into UCS2
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: iso8859.c,v $
 * Revision 1.2  1995/03/08  14:09:44  rj
 * tables added for ISO 8859-2 .. -10.
 *
 * Revision 1.1.1.1  1993/12/31  20:56:44  rj
 * erster cvs import.
 *
 */

static const char RCSId[] = "$Id: iso8859.c,v 1.2 1995/03/08 14:09:44 rj Exp $";

#include "psfilt.h"
#include "ucs2list.h"
#include "iso8859.h"

/* ATTENTION: codes 0x00 - 0xa0 are omitted! (codes 0x00 - 0x7f are identical to ASCII, so see iso646.c) */

static t_ucs2 ISOlatin1_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_inverted_exclamation_mark,
  UCS2_cent_sign,
  UCS2_pound_sign,
  UCS2_currency_sign,
  UCS2_yen_sign,
  UCS2_broken_bar,
  UCS2_section_sign,
  UCS2_diaeresis,
  UCS2_copyright_sign,
  UCS2_feminine_ordinal_indicator,
  UCS2_left_pointing_double_angle_quotation_mark,
  UCS2_not_sign,
  UCS2_soft_hyphen,
  UCS2_registered_sign,
  UCS2_macron,
  UCS2_degree_sign,
  UCS2_plus_minus_sign,
  UCS2_superscript_two,
  UCS2_superscript_three,
  UCS2_acute_accent,
  UCS2_micro_sign,
  UCS2_pilcrow_sign,
  UCS2_middle_dot,
  UCS2_cedilla,
  UCS2_superscript_one,
  UCS2_masculine_ordinal_indicator,
  UCS2_right_pointing_double_angle_quotation_mark,
  UCS2_vulgar_fraction_one_quarter,
  UCS2_vulgar_fraction_one_half,
  UCS2_vulgar_fraction_three_quarters,
  UCS2_inverted_question_mark,
  UCS2_latin_capital_letter_a_with_grave,
  UCS2_latin_capital_letter_a_with_acute,
  UCS2_latin_capital_letter_a_with_circumflex,
  UCS2_latin_capital_letter_a_with_tilde,
  UCS2_latin_capital_letter_a_with_diaeresis,
  UCS2_latin_capital_letter_a_with_ring_above,
  UCS2_latin_capital_ligature_ae,
  UCS2_latin_capital_letter_c_with_cedilla,
  UCS2_latin_capital_letter_e_with_grave,
  UCS2_latin_capital_letter_e_with_acute,
  UCS2_latin_capital_letter_e_with_circumflex,
  UCS2_latin_capital_letter_e_with_diaeresis,
  UCS2_latin_capital_letter_i_with_grave,
  UCS2_latin_capital_letter_i_with_acute,
  UCS2_latin_capital_letter_i_with_circumflex,
  UCS2_latin_capital_letter_i_with_diaeresis,
  UCS2_latin_capital_letter_eth,
  UCS2_latin_capital_letter_n_with_tilde,
  UCS2_latin_capital_letter_o_with_grave,
  UCS2_latin_capital_letter_o_with_acute,
  UCS2_latin_capital_letter_o_with_circumflex,
  UCS2_latin_capital_letter_o_with_tilde,
  UCS2_latin_capital_letter_o_with_diaeresis,
  UCS2_multiplication_sign,
  UCS2_latin_capital_letter_o_with_stroke,
  UCS2_latin_capital_letter_u_with_grave,
  UCS2_latin_capital_letter_u_with_acute,
  UCS2_latin_capital_letter_u_with_circumflex,
  UCS2_latin_capital_letter_u_with_diaeresis,
  UCS2_latin_capital_letter_y_with_acute,
  UCS2_latin_capital_letter_thorn,
  UCS2_latin_small_letter_sharp_s,
  UCS2_latin_small_letter_a_with_grave,
  UCS2_latin_small_letter_a_with_acute,
  UCS2_latin_small_letter_a_with_circumflex,
  UCS2_latin_small_letter_a_with_tilde,
  UCS2_latin_small_letter_a_with_diaeresis,
  UCS2_latin_small_letter_a_with_ring_above,
  UCS2_latin_small_ligature_ae,
  UCS2_latin_small_letter_c_with_cedilla,
  UCS2_latin_small_letter_e_with_grave,
  UCS2_latin_small_letter_e_with_acute,
  UCS2_latin_small_letter_e_with_circumflex,
  UCS2_latin_small_letter_e_with_diaeresis,
  UCS2_latin_small_letter_i_with_grave,
  UCS2_latin_small_letter_i_with_acute,
  UCS2_latin_small_letter_i_with_circumflex,
  UCS2_latin_small_letter_i_with_diaeresis,
  UCS2_latin_small_letter_eth,
  UCS2_latin_small_letter_n_with_tilde,
  UCS2_latin_small_letter_o_with_grave,
  UCS2_latin_small_letter_o_with_acute,
  UCS2_latin_small_letter_o_with_circumflex,
  UCS2_latin_small_letter_o_with_tilde,
  UCS2_latin_small_letter_o_with_diaeresis,
  UCS2_division_sign,
  UCS2_latin_small_letter_o_with_stroke,
  UCS2_latin_small_letter_u_with_grave,
  UCS2_latin_small_letter_u_with_acute,
  UCS2_latin_small_letter_u_with_circumflex,
  UCS2_latin_small_letter_u_with_diaeresis,
  UCS2_latin_small_letter_y_with_acute,
  UCS2_latin_small_letter_thorn,
  UCS2_latin_small_letter_y_with_diaeresis,
};

t_ucs2 ISOlatin1toUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOlatin1_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOlatin2_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_latin_capital_letter_a_with_ogonek,
  UCS2_breve,
  UCS2_latin_capital_letter_l_with_stroke,
  UCS2_currency_sign,
  UCS2_latin_capital_letter_l_with_caron,
  UCS2_latin_capital_letter_s_with_acute,
  UCS2_section_sign,
  UCS2_diaeresis,
  UCS2_latin_capital_letter_s_with_caron,
  UCS2_latin_capital_letter_s_with_cedilla,
  UCS2_latin_capital_letter_t_with_caron,
  UCS2_latin_capital_letter_z_with_acute,
  UCS2_soft_hyphen,
  UCS2_latin_capital_letter_z_with_caron,
  UCS2_latin_capital_letter_z_with_dot_above,
  UCS2_degree_sign,
  UCS2_latin_small_letter_a_with_ogonek,
  UCS2_ogonek,
  UCS2_latin_small_letter_l_with_stroke,
  UCS2_acute_accent,
  UCS2_latin_small_letter_l_with_caron,
  UCS2_latin_small_letter_s_with_acute,
  UCS2_caron,
  UCS2_cedilla,
  UCS2_latin_small_letter_s_with_caron,
  UCS2_latin_small_letter_s_with_cedilla,
  UCS2_latin_small_letter_t_with_caron,
  UCS2_latin_small_letter_z_with_acute,
  UCS2_double_acute_accent,
  UCS2_latin_small_letter_z_with_caron,
  UCS2_latin_small_letter_z_with_dot_above,
  UCS2_latin_capital_letter_r_with_acute,
  UCS2_latin_capital_letter_a_with_acute,
  UCS2_latin_capital_letter_a_with_circumflex,
  UCS2_latin_capital_letter_a_with_breve,
  UCS2_latin_capital_letter_a_with_diaeresis,
  UCS2_latin_capital_letter_l_with_acute,
  UCS2_latin_capital_letter_c_with_acute,
  UCS2_latin_capital_letter_c_with_cedilla,
  UCS2_latin_capital_letter_c_with_caron,
  UCS2_latin_capital_letter_e_with_acute,
  UCS2_latin_capital_letter_e_with_ogonek,
  UCS2_latin_capital_letter_e_with_diaeresis,
  UCS2_latin_capital_letter_e_with_caron,
  UCS2_latin_capital_letter_i_with_acute,
  UCS2_latin_capital_letter_i_with_circumflex,
  UCS2_latin_capital_letter_d_with_caron,
  UCS2_latin_capital_letter_d_with_stroke,
  UCS2_latin_capital_letter_n_with_acute,
  UCS2_latin_capital_letter_n_with_caron,
  UCS2_latin_capital_letter_o_with_acute,
  UCS2_latin_capital_letter_o_with_circumflex,
  UCS2_latin_capital_letter_o_with_double_acute,
  UCS2_latin_capital_letter_o_with_diaeresis,
  UCS2_multiplication_sign,
  UCS2_latin_capital_letter_r_with_caron,
  UCS2_latin_capital_letter_u_with_ring_above,
  UCS2_latin_capital_letter_u_with_acute,
  UCS2_latin_capital_letter_u_with_double_acute,
  UCS2_latin_capital_letter_u_with_diaeresis,
  UCS2_latin_capital_letter_y_with_acute,
  UCS2_latin_capital_letter_t_with_cedilla,
  UCS2_latin_small_letter_sharp_s,
  UCS2_latin_small_letter_r_with_acute,
  UCS2_latin_small_letter_a_with_acute,
  UCS2_latin_small_letter_a_with_circumflex,
  UCS2_latin_small_letter_a_with_breve,
  UCS2_latin_small_letter_a_with_diaeresis,
  UCS2_latin_small_letter_l_with_acute,
  UCS2_latin_small_letter_c_with_acute,
  UCS2_latin_small_letter_c_with_cedilla,
  UCS2_latin_small_letter_c_with_caron,
  UCS2_latin_small_letter_e_with_acute,
  UCS2_latin_small_letter_e_with_ogonek,
  UCS2_latin_small_letter_e_with_diaeresis,
  UCS2_latin_small_letter_e_with_caron,
  UCS2_latin_small_letter_i_with_acute,
  UCS2_latin_small_letter_i_with_circumflex,
  UCS2_latin_small_letter_d_with_caron,
  UCS2_latin_small_letter_d_with_stroke,
  UCS2_latin_small_letter_n_with_acute,
  UCS2_latin_small_letter_n_with_caron,
  UCS2_latin_small_letter_o_with_acute,
  UCS2_latin_small_letter_o_with_circumflex,
  UCS2_latin_small_letter_o_with_double_acute,
  UCS2_latin_small_letter_o_with_diaeresis,
  UCS2_division_sign,
  UCS2_latin_small_letter_r_with_caron,
  UCS2_latin_small_letter_u_with_ring_above,
  UCS2_latin_small_letter_u_with_acute,
  UCS2_latin_small_letter_u_with_double_acute,
  UCS2_latin_small_letter_u_with_diaeresis,
  UCS2_latin_small_letter_y_with_acute,
  UCS2_latin_small_letter_t_with_cedilla,
  UCS2_dot_above,
};

t_ucs2 ISOlatin2toUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOlatin2_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOlatin3_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_latin_capital_letter_h_with_stroke,
  UCS2_breve,
  UCS2_pound_sign,
  UCS2_currency_sign,
  UCS2_VOID,
  UCS2_latin_capital_letter_h_with_circumflex,
  UCS2_section_sign,
  UCS2_diaeresis,
  UCS2_latin_capital_letter_i_with_dot_above,
  UCS2_latin_capital_letter_s_with_cedilla,
  UCS2_latin_capital_letter_g_with_breve,
  UCS2_latin_capital_letter_j_with_circumflex,
  UCS2_soft_hyphen,
  UCS2_VOID,
  UCS2_latin_capital_letter_z_with_dot_above,
  UCS2_degree_sign,
  UCS2_latin_small_letter_h_with_stroke,
  UCS2_superscript_two,
  UCS2_superscript_three,
  UCS2_acute_accent,
  UCS2_micro_sign,
  UCS2_latin_small_letter_h_with_circumflex,
  UCS2_middle_dot,
  UCS2_cedilla,
  UCS2_latin_small_letter_dotless_i,
  UCS2_latin_small_letter_s_with_cedilla,
  UCS2_latin_small_letter_g_with_breve,
  UCS2_latin_small_letter_j_with_circumflex,
  UCS2_vulgar_fraction_one_half,
  UCS2_VOID,
  UCS2_latin_small_letter_z_with_dot_above,
  UCS2_latin_capital_letter_a_with_grave,
  UCS2_latin_capital_letter_a_with_acute,
  UCS2_latin_capital_letter_a_with_circumflex,
  UCS2_VOID,
  UCS2_latin_capital_letter_a_with_diaeresis,
  UCS2_latin_capital_letter_c_with_dot_above,
  UCS2_latin_capital_letter_c_with_circumflex,
  UCS2_latin_capital_letter_c_with_cedilla,
  UCS2_latin_capital_letter_e_with_grave,
  UCS2_latin_capital_letter_e_with_acute,
  UCS2_latin_capital_letter_e_with_circumflex,
  UCS2_latin_capital_letter_e_with_diaeresis,
  UCS2_latin_capital_letter_i_with_grave,
  UCS2_latin_capital_letter_i_with_acute,
  UCS2_latin_capital_letter_i_with_circumflex,
  UCS2_latin_capital_letter_i_with_diaeresis,
  UCS2_VOID,
  UCS2_latin_capital_letter_n_with_tilde,
  UCS2_latin_capital_letter_o_with_grave,
  UCS2_latin_capital_letter_o_with_acute,
  UCS2_latin_capital_letter_o_with_circumflex,
  UCS2_latin_capital_letter_g_with_dot_above,
  UCS2_latin_capital_letter_o_with_diaeresis,
  UCS2_multiplication_sign,
  UCS2_latin_capital_letter_g_with_circumflex,
  UCS2_latin_capital_letter_u_with_grave,
  UCS2_latin_capital_letter_u_with_acute,
  UCS2_latin_capital_letter_u_with_circumflex,
  UCS2_latin_capital_letter_u_with_diaeresis,
  UCS2_latin_capital_letter_u_with_breve,
  UCS2_latin_capital_letter_s_with_circumflex,
  UCS2_latin_small_letter_sharp_s,
  UCS2_latin_small_letter_a_with_grave,
  UCS2_latin_small_letter_a_with_acute,
  UCS2_latin_small_letter_a_with_circumflex,
  UCS2_VOID,
  UCS2_latin_small_letter_a_with_diaeresis,
  UCS2_latin_small_letter_c_with_dot_above,
  UCS2_latin_small_letter_c_with_circumflex,
  UCS2_latin_small_letter_c_with_cedilla,
  UCS2_latin_small_letter_e_with_grave,
  UCS2_latin_small_letter_e_with_acute,
  UCS2_latin_small_letter_e_with_circumflex,
  UCS2_latin_small_letter_e_with_diaeresis,
  UCS2_latin_small_letter_i_with_grave,
  UCS2_latin_small_letter_i_with_acute,
  UCS2_latin_small_letter_i_with_circumflex,
  UCS2_latin_small_letter_i_with_diaeresis,
  UCS2_VOID,
  UCS2_latin_small_letter_n_with_tilde,
  UCS2_latin_small_letter_o_with_grave,
  UCS2_latin_small_letter_o_with_acute,
  UCS2_latin_small_letter_o_with_circumflex,
  UCS2_latin_small_letter_g_with_dot_above,
  UCS2_latin_small_letter_o_with_diaeresis,
  UCS2_division_sign,
  UCS2_latin_small_letter_g_with_circumflex,
  UCS2_latin_small_letter_u_with_grave,
  UCS2_latin_small_letter_u_with_acute,
  UCS2_latin_small_letter_u_with_circumflex,
  UCS2_latin_small_letter_u_with_diaeresis,
  UCS2_latin_small_letter_u_with_breve,
  UCS2_latin_small_letter_s_with_circumflex,
  UCS2_dot_above,
};

t_ucs2 ISOlatin3toUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOlatin3_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOlatin4_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_latin_capital_letter_a_with_ogonek,
  UCS2_latin_small_letter_kra,
  UCS2_latin_capital_letter_r_with_cedilla,
  UCS2_currency_sign,
  UCS2_latin_capital_letter_i_with_tilde,
  UCS2_latin_capital_letter_l_with_cedilla,
  UCS2_section_sign,
  UCS2_diaeresis,
  UCS2_latin_capital_letter_s_with_caron,
  UCS2_latin_capital_letter_e_with_macron,
  UCS2_latin_capital_letter_g_with_cedilla,
  UCS2_latin_capital_letter_t_with_stroke,
  UCS2_soft_hyphen,
  UCS2_latin_capital_letter_z_with_caron,
  UCS2_macron,
  UCS2_degree_sign,
  UCS2_latin_small_letter_a_with_ogonek,
  UCS2_ogonek,
  UCS2_latin_small_letter_r_with_cedilla,
  UCS2_acute_accent,
  UCS2_latin_small_letter_i_with_tilde,
  UCS2_latin_small_letter_l_with_cedilla,
  UCS2_caron,
  UCS2_cedilla,
  UCS2_latin_small_letter_s_with_caron,
  UCS2_latin_small_letter_e_with_macron,
  UCS2_latin_small_letter_g_with_cedilla,
  UCS2_latin_small_letter_t_with_stroke,
  UCS2_latin_capital_letter_eng,
  UCS2_latin_small_letter_z_with_caron,
  UCS2_latin_small_letter_eng,
  UCS2_latin_capital_letter_a_with_macron,
  UCS2_latin_capital_letter_a_with_acute,
  UCS2_latin_capital_letter_a_with_circumflex,
  UCS2_latin_capital_letter_a_with_tilde,
  UCS2_latin_capital_letter_a_with_diaeresis,
  UCS2_latin_capital_letter_a_with_ring_above,
  UCS2_latin_capital_ligature_ae,
  UCS2_latin_capital_letter_i_with_ogonek,
  UCS2_latin_capital_letter_c_with_caron,
  UCS2_latin_capital_letter_e_with_acute,
  UCS2_latin_capital_letter_e_with_ogonek,
  UCS2_latin_capital_letter_e_with_diaeresis,
  UCS2_latin_capital_letter_e_with_dot_above,
  UCS2_latin_capital_letter_i_with_acute,
  UCS2_latin_capital_letter_i_with_circumflex,
  UCS2_latin_capital_letter_i_with_macron,
  UCS2_latin_capital_letter_d_with_stroke,
  UCS2_latin_capital_letter_n_with_cedilla,
  UCS2_latin_capital_letter_o_with_macron,
  UCS2_latin_capital_letter_k_with_cedilla,
  UCS2_latin_capital_letter_o_with_circumflex,
  UCS2_latin_capital_letter_o_with_tilde,
  UCS2_latin_capital_letter_o_with_diaeresis,
  UCS2_multiplication_sign,
  UCS2_latin_capital_letter_o_with_stroke,
  UCS2_latin_capital_letter_u_with_ogonek,
  UCS2_latin_capital_letter_u_with_acute,
  UCS2_latin_capital_letter_u_with_circumflex,
  UCS2_latin_capital_letter_u_with_diaeresis,
  UCS2_latin_capital_letter_u_with_tilde,
  UCS2_latin_capital_letter_u_with_macron,
  UCS2_latin_small_letter_sharp_s,
  UCS2_latin_small_letter_a_with_macron,
  UCS2_latin_small_letter_a_with_acute,
  UCS2_latin_small_letter_a_with_circumflex,
  UCS2_latin_small_letter_a_with_tilde,
  UCS2_latin_small_letter_a_with_diaeresis,
  UCS2_latin_small_letter_a_with_ring_above,
  UCS2_latin_small_ligature_ae,
  UCS2_latin_small_letter_i_with_ogonek,
  UCS2_latin_small_letter_c_with_caron,
  UCS2_latin_small_letter_e_with_acute,
  UCS2_latin_small_letter_e_with_ogonek,
  UCS2_latin_small_letter_e_with_diaeresis,
  UCS2_latin_small_letter_e_with_dot_above,
  UCS2_latin_small_letter_i_with_acute,
  UCS2_latin_small_letter_i_with_circumflex,
  UCS2_latin_small_letter_i_with_macron,
  UCS2_latin_small_letter_d_with_stroke,
  UCS2_latin_small_letter_n_with_cedilla,
  UCS2_latin_small_letter_o_with_macron,
  UCS2_latin_small_letter_k_with_cedilla,
  UCS2_latin_small_letter_o_with_circumflex,
  UCS2_latin_small_letter_o_with_tilde,
  UCS2_latin_small_letter_o_with_diaeresis,
  UCS2_division_sign,
  UCS2_latin_small_letter_o_with_stroke,
  UCS2_latin_small_letter_u_with_ogonek,
  UCS2_latin_small_letter_u_with_acute,
  UCS2_latin_small_letter_u_with_circumflex,
  UCS2_latin_small_letter_u_with_diaeresis,
  UCS2_latin_small_letter_u_with_tilde,
  UCS2_latin_small_letter_u_with_macron,
  UCS2_dot_above,
};

t_ucs2 ISOlatin4toUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOlatin4_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOcyrillic_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_cyrillic_capital_letter_io,
  UCS2_cyrillic_capital_letter_dje,
  UCS2_cyrillic_capital_letter_gje,
  UCS2_cyrillic_capital_letter_ukrainian_ie,
  UCS2_cyrillic_capital_letter_dze,
  UCS2_cyrillic_capital_letter_byelorussian_ukrainian_i,
  UCS2_cyrillic_capital_letter_yi,
  UCS2_cyrillic_capital_letter_je,
  UCS2_cyrillic_capital_letter_lje,
  UCS2_cyrillic_capital_letter_nje,
  UCS2_cyrillic_capital_letter_tshe,
  UCS2_cyrillic_capital_letter_kje,
  UCS2_soft_hyphen,
  UCS2_cyrillic_capital_letter_short_u,
  UCS2_cyrillic_capital_letter_dzhe,
  UCS2_cyrillic_capital_letter_a,
  UCS2_cyrillic_capital_letter_be,
  UCS2_cyrillic_capital_letter_ve,
  UCS2_cyrillic_capital_letter_ghe,
  UCS2_cyrillic_capital_letter_de,
  UCS2_cyrillic_capital_letter_ie,
  UCS2_cyrillic_capital_letter_zhe,
  UCS2_cyrillic_capital_letter_ze,
  UCS2_cyrillic_capital_letter_i,
  UCS2_cyrillic_capital_letter_short_i,
  UCS2_cyrillic_capital_letter_ka,
  UCS2_cyrillic_capital_letter_el,
  UCS2_cyrillic_capital_letter_em,
  UCS2_cyrillic_capital_letter_en,
  UCS2_cyrillic_capital_letter_o,
  UCS2_cyrillic_capital_letter_pe,
  UCS2_cyrillic_capital_letter_er,
  UCS2_cyrillic_capital_letter_es,
  UCS2_cyrillic_capital_letter_te,
  UCS2_cyrillic_capital_letter_u,
  UCS2_cyrillic_capital_letter_ef,
  UCS2_cyrillic_capital_letter_ha,
  UCS2_cyrillic_capital_letter_tse,
  UCS2_cyrillic_capital_letter_che,
  UCS2_cyrillic_capital_letter_sha,
  UCS2_cyrillic_capital_letter_shcha,
  UCS2_cyrillic_capital_letter_hard_sign,
  UCS2_cyrillic_capital_letter_yeru,
  UCS2_cyrillic_capital_letter_soft_sign,
  UCS2_cyrillic_capital_letter_e,
  UCS2_cyrillic_capital_letter_yu,
  UCS2_cyrillic_capital_letter_ya,
  UCS2_cyrillic_small_letter_a,
  UCS2_cyrillic_small_letter_be,
  UCS2_cyrillic_small_letter_ve,
  UCS2_cyrillic_small_letter_ghe,
  UCS2_cyrillic_small_letter_de,
  UCS2_cyrillic_small_letter_ie,
  UCS2_cyrillic_small_letter_zhe,
  UCS2_cyrillic_small_letter_ze,
  UCS2_cyrillic_small_letter_i,
  UCS2_cyrillic_small_letter_short_i,
  UCS2_cyrillic_small_letter_ka,
  UCS2_cyrillic_small_letter_el,
  UCS2_cyrillic_small_letter_em,
  UCS2_cyrillic_small_letter_en,
  UCS2_cyrillic_small_letter_o,
  UCS2_cyrillic_small_letter_pe,
  UCS2_cyrillic_small_letter_er,
  UCS2_cyrillic_small_letter_es,
  UCS2_cyrillic_small_letter_te,
  UCS2_cyrillic_small_letter_u,
  UCS2_cyrillic_small_letter_ef,
  UCS2_cyrillic_small_letter_ha,
  UCS2_cyrillic_small_letter_tse,
  UCS2_cyrillic_small_letter_che,
  UCS2_cyrillic_small_letter_sha,
  UCS2_cyrillic_small_letter_shcha,
  UCS2_cyrillic_small_letter_hard_sign,
  UCS2_cyrillic_small_letter_yeru,
  UCS2_cyrillic_small_letter_soft_sign,
  UCS2_cyrillic_small_letter_e,
  UCS2_cyrillic_small_letter_yu,
  UCS2_cyrillic_small_letter_ya,
  UCS2_numero_sign,
  UCS2_cyrillic_small_letter_io,
  UCS2_cyrillic_small_letter_dje,
  UCS2_cyrillic_small_letter_gje,
  UCS2_cyrillic_small_letter_ukrainian_ie,
  UCS2_cyrillic_small_letter_dze,
  UCS2_cyrillic_small_letter_byelorussian_ukrainian_i,
  UCS2_cyrillic_small_letter_yi,
  UCS2_cyrillic_small_letter_je,
  UCS2_cyrillic_small_letter_lje,
  UCS2_cyrillic_small_letter_nje,
  UCS2_cyrillic_small_letter_tshe,
  UCS2_cyrillic_small_letter_kje,
  UCS2_section_sign,
  UCS2_cyrillic_small_letter_short_u,
  UCS2_cyrillic_small_letter_dzhe,
};

t_ucs2 ISOcyrillictoUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOcyrillic_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOarabic_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_currency_sign,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_arabic_comma,
  UCS2_soft_hyphen,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_arabic_semicolon,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_arabic_question_mark,
  UCS2_VOID,
  UCS2_arabic_letter_hamza,
  UCS2_arabic_letter_alef_with_madda_above,
  UCS2_arabic_letter_alef_with_hamza_above,
  UCS2_arabic_letter_waw_with_hamza_above,
  UCS2_arabic_letter_alef_with_hamza_below,
  UCS2_arabic_letter_yeh_with_hamza_above,
  UCS2_arabic_letter_alef,
  UCS2_arabic_letter_beh,
  UCS2_arabic_letter_teh_marbuta,
  UCS2_arabic_letter_teh,
  UCS2_arabic_letter_theh,
  UCS2_arabic_letter_jeem,
  UCS2_arabic_letter_hah,
  UCS2_arabic_letter_khah,
  UCS2_arabic_letter_dal,
  UCS2_arabic_letter_thal,
  UCS2_arabic_letter_reh,
  UCS2_arabic_letter_zain,
  UCS2_arabic_letter_seen,
  UCS2_arabic_letter_sheen,
  UCS2_arabic_letter_sad,
  UCS2_arabic_letter_dad,
  UCS2_arabic_letter_tah,
  UCS2_arabic_letter_zah,
  UCS2_arabic_letter_ain,
  UCS2_arabic_letter_ghain,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_arabic_tatweel,
  UCS2_arabic_letter_feh,
  UCS2_arabic_letter_qaf,
  UCS2_arabic_letter_kaf,
  UCS2_arabic_letter_lam,
  UCS2_arabic_letter_meem,
  UCS2_arabic_letter_noon,
  UCS2_arabic_letter_heh,
  UCS2_arabic_letter_waw,
  UCS2_arabic_letter_alef_maksura,
  UCS2_arabic_letter_yeh,
  UCS2_arabic_fathatan,
  UCS2_arabic_dammatan,
  UCS2_arabic_kasratan,
  UCS2_arabic_fatha,
  UCS2_arabic_damma,
  UCS2_arabic_kasra,
  UCS2_arabic_shadda,
  UCS2_arabic_sukun,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
};

t_ucs2 ISOarabictoUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOarabic_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOgreek_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_left_single_quotation_mark,
  UCS2_right_single_quotation_mark,
  UCS2_pound_sign,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_broken_bar,
  UCS2_section_sign,
  UCS2_diaeresis,
  UCS2_copyright_sign,
  UCS2_VOID,
  UCS2_left_pointing_double_angle_quotation_mark,
  UCS2_not_sign,
  UCS2_soft_hyphen,
  UCS2_VOID,
  UCS2_horizontal_bar,
  UCS2_degree_sign,
  UCS2_plus_minus_sign,
  UCS2_superscript_two,
  UCS2_superscript_three,
  UCS2_greek_tonos,
  UCS2_greek_dialytika_tonos,
  UCS2_greek_capital_letter_alpha_with_tonos,
  UCS2_middle_dot,
  UCS2_greek_capital_letter_epsilon_with_tonos,
  UCS2_greek_capital_letter_eta_with_tonos,
  UCS2_greek_capital_letter_iota_with_tonos,
  UCS2_right_pointing_double_angle_quotation_mark,
  UCS2_greek_capital_letter_omicron_with_tonos,
  UCS2_vulgar_fraction_one_half,
  UCS2_greek_capital_letter_upsilon_with_tonos,
  UCS2_greek_capital_letter_omega_with_tonos,
  UCS2_greek_small_letter_iota_with_dialytika_and_tonos,
  UCS2_greek_capital_letter_alpha,
  UCS2_greek_capital_letter_beta,
  UCS2_greek_capital_letter_gamma,
  UCS2_greek_capital_letter_delta,
  UCS2_greek_capital_letter_epsilon,
  UCS2_greek_capital_letter_zeta,
  UCS2_greek_capital_letter_eta,
  UCS2_greek_capital_letter_theta,
  UCS2_greek_capital_letter_iota,
  UCS2_greek_capital_letter_kappa,
  UCS2_greek_capital_letter_lamda,
  UCS2_greek_capital_letter_mu,
  UCS2_greek_capital_letter_nu,
  UCS2_greek_capital_letter_xi,
  UCS2_greek_capital_letter_omicron,
  UCS2_greek_capital_letter_pi,
  UCS2_greek_capital_letter_rho,
  UCS2_VOID,
  UCS2_greek_capital_letter_sigma,
  UCS2_greek_capital_letter_tau,
  UCS2_greek_capital_letter_upsilon,
  UCS2_greek_capital_letter_phi,
  UCS2_greek_capital_letter_chi,
  UCS2_greek_capital_letter_psi,
  UCS2_greek_capital_letter_omega,
  UCS2_greek_capital_letter_iota_with_dialytika,
  UCS2_greek_capital_letter_upsilon_with_dialytika,
  UCS2_greek_small_letter_alpha_with_tonos,
  UCS2_greek_small_letter_epsilon_with_tonos,
  UCS2_greek_small_letter_eta_with_tonos,
  UCS2_greek_small_letter_iota_with_tonos,
  UCS2_greek_small_letter_upsilon_with_dialytika_and_tonos,
  UCS2_greek_small_letter_alpha,
  UCS2_greek_small_letter_beta,
  UCS2_greek_small_letter_gamma,
  UCS2_greek_small_letter_delta,
  UCS2_greek_small_letter_epsilon,
  UCS2_greek_small_letter_zeta,
  UCS2_greek_small_letter_eta,
  UCS2_greek_small_letter_theta,
  UCS2_greek_small_letter_iota,
  UCS2_greek_small_letter_kappa,
  UCS2_greek_small_letter_lamda,
  UCS2_greek_small_letter_mu,
  UCS2_greek_small_letter_nu,
  UCS2_greek_small_letter_xi,
  UCS2_greek_small_letter_omicron,
  UCS2_greek_small_letter_pi,
  UCS2_greek_small_letter_rho,
  UCS2_greek_small_letter_final_sigma,
  UCS2_greek_small_letter_sigma,
  UCS2_greek_small_letter_tau,
  UCS2_greek_small_letter_upsilon,
  UCS2_greek_small_letter_phi,
  UCS2_greek_small_letter_chi,
  UCS2_greek_small_letter_psi,
  UCS2_greek_small_letter_omega,
  UCS2_greek_small_letter_iota_with_dialytika,
  UCS2_greek_small_letter_upsilon_with_dialytika,
  UCS2_greek_small_letter_omicron_with_tonos,
  UCS2_greek_small_letter_upsilon_with_tonos,
  UCS2_greek_small_letter_omega_with_tonos,
  UCS2_VOID,
};

t_ucs2 ISOgreektoUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOgreek_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOhebrew_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_VOID,
  UCS2_cent_sign,
  UCS2_pound_sign,
  UCS2_currency_sign,
  UCS2_yen_sign,
  UCS2_broken_bar,
  UCS2_section_sign,
  UCS2_diaeresis,
  UCS2_copyright_sign,
  UCS2_multiplication_sign,
  UCS2_left_pointing_double_angle_quotation_mark,
  UCS2_not_sign,
  UCS2_soft_hyphen,
  UCS2_registered_sign,
  UCS2_overline,
  UCS2_degree_sign,
  UCS2_plus_minus_sign,
  UCS2_superscript_two,
  UCS2_superscript_three,
  UCS2_acute_accent,
  UCS2_micro_sign,
  UCS2_pilcrow_sign,
  UCS2_middle_dot,
  UCS2_cedilla,
  UCS2_superscript_one,
  UCS2_division_sign,
  UCS2_right_pointing_double_angle_quotation_mark,
  UCS2_vulgar_fraction_one_quarter,
  UCS2_vulgar_fraction_one_half,
  UCS2_vulgar_fraction_three_quarters,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_double_low_line,
  UCS2_hebrew_letter_alef,
  UCS2_hebrew_letter_bet,
  UCS2_hebrew_letter_gimel,
  UCS2_hebrew_letter_dalet,
  UCS2_hebrew_letter_he,
  UCS2_hebrew_letter_vav,
  UCS2_hebrew_letter_zayin,
  UCS2_hebrew_letter_het,
  UCS2_hebrew_letter_tet,
  UCS2_hebrew_letter_yod,
  UCS2_hebrew_letter_final_kaf,
  UCS2_hebrew_letter_kaf,
  UCS2_hebrew_letter_lamed,
  UCS2_hebrew_letter_final_mem,
  UCS2_hebrew_letter_mem,
  UCS2_hebrew_letter_final_nun,
  UCS2_hebrew_letter_nun,
  UCS2_hebrew_letter_samekh,
  UCS2_hebrew_letter_ayin,
  UCS2_hebrew_letter_final_pe,
  UCS2_hebrew_letter_pe,
  UCS2_hebrew_letter_final_tsadi,
  UCS2_hebrew_letter_tsadi,
  UCS2_hebrew_letter_qof,
  UCS2_hebrew_letter_resh,
  UCS2_hebrew_letter_shin,
  UCS2_hebrew_letter_tav,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
  UCS2_VOID,
};

t_ucs2 ISOhebrewtoUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOhebrew_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOlatin5_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_inverted_exclamation_mark,
  UCS2_cent_sign,
  UCS2_pound_sign,
  UCS2_currency_sign,
  UCS2_yen_sign,
  UCS2_broken_bar,
  UCS2_section_sign,
  UCS2_diaeresis,
  UCS2_copyright_sign,
  UCS2_feminine_ordinal_indicator,
  UCS2_left_pointing_double_angle_quotation_mark,
  UCS2_not_sign,
  UCS2_soft_hyphen,
  UCS2_registered_sign,
  UCS2_macron,
  UCS2_degree_sign,
  UCS2_plus_minus_sign,
  UCS2_superscript_two,
  UCS2_superscript_three,
  UCS2_acute_accent,
  UCS2_micro_sign,
  UCS2_pilcrow_sign,
  UCS2_middle_dot,
  UCS2_cedilla,
  UCS2_superscript_one,
  UCS2_masculine_ordinal_indicator,
  UCS2_right_pointing_double_angle_quotation_mark,
  UCS2_vulgar_fraction_one_quarter,
  UCS2_vulgar_fraction_one_half,
  UCS2_vulgar_fraction_three_quarters,
  UCS2_inverted_question_mark,
  UCS2_latin_capital_letter_a_with_grave,
  UCS2_latin_capital_letter_a_with_acute,
  UCS2_latin_capital_letter_a_with_circumflex,
  UCS2_latin_capital_letter_a_with_tilde,
  UCS2_latin_capital_letter_a_with_diaeresis,
  UCS2_latin_capital_letter_a_with_ring_above,
  UCS2_latin_capital_ligature_ae,
  UCS2_latin_capital_letter_c_with_cedilla,
  UCS2_latin_capital_letter_e_with_grave,
  UCS2_latin_capital_letter_e_with_acute,
  UCS2_latin_capital_letter_e_with_circumflex,
  UCS2_latin_capital_letter_e_with_diaeresis,
  UCS2_latin_capital_letter_i_with_grave,
  UCS2_latin_capital_letter_i_with_acute,
  UCS2_latin_capital_letter_i_with_circumflex,
  UCS2_latin_capital_letter_i_with_diaeresis,
  UCS2_latin_capital_letter_g_with_breve,
  UCS2_latin_capital_letter_n_with_tilde,
  UCS2_latin_capital_letter_o_with_grave,
  UCS2_latin_capital_letter_o_with_acute,
  UCS2_latin_capital_letter_o_with_circumflex,
  UCS2_latin_capital_letter_o_with_tilde,
  UCS2_latin_capital_letter_o_with_diaeresis,
  UCS2_multiplication_sign,
  UCS2_latin_capital_letter_o_with_stroke,
  UCS2_latin_capital_letter_u_with_grave,
  UCS2_latin_capital_letter_u_with_acute,
  UCS2_latin_capital_letter_u_with_circumflex,
  UCS2_latin_capital_letter_u_with_diaeresis,
  UCS2_latin_capital_letter_i_with_dot_above,
  UCS2_latin_capital_letter_s_with_cedilla,
  UCS2_latin_small_letter_sharp_s,
  UCS2_latin_small_letter_a_with_grave,
  UCS2_latin_small_letter_a_with_acute,
  UCS2_latin_small_letter_a_with_circumflex,
  UCS2_latin_small_letter_a_with_tilde,
  UCS2_latin_small_letter_a_with_diaeresis,
  UCS2_latin_small_letter_a_with_ring_above,
  UCS2_latin_small_ligature_ae,
  UCS2_latin_small_letter_c_with_cedilla,
  UCS2_latin_small_letter_e_with_grave,
  UCS2_latin_small_letter_e_with_acute,
  UCS2_latin_small_letter_e_with_circumflex,
  UCS2_latin_small_letter_e_with_diaeresis,
  UCS2_latin_small_letter_i_with_grave,
  UCS2_latin_small_letter_i_with_acute,
  UCS2_latin_small_letter_i_with_circumflex,
  UCS2_latin_small_letter_i_with_diaeresis,
  UCS2_latin_small_letter_g_with_breve,
  UCS2_latin_small_letter_n_with_tilde,
  UCS2_latin_small_letter_o_with_grave,
  UCS2_latin_small_letter_o_with_acute,
  UCS2_latin_small_letter_o_with_circumflex,
  UCS2_latin_small_letter_o_with_tilde,
  UCS2_latin_small_letter_o_with_diaeresis,
  UCS2_division_sign,
  UCS2_latin_small_letter_o_with_stroke,
  UCS2_latin_small_letter_u_with_grave,
  UCS2_latin_small_letter_u_with_acute,
  UCS2_latin_small_letter_u_with_circumflex,
  UCS2_latin_small_letter_u_with_diaeresis,
  UCS2_latin_small_letter_dotless_i,
  UCS2_latin_small_letter_s_with_cedilla,
  UCS2_latin_small_letter_y_with_diaeresis,
};

t_ucs2 ISOlatin5toUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOlatin5_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}

static t_ucs2 ISOsami_to_UCS2_table[] =
{
  UCS2_no_break_space,
  UCS2_latin_capital_letter_a_with_ogonek,
  UCS2_latin_capital_letter_e_with_macron,
  UCS2_latin_capital_letter_g_with_cedilla,
  UCS2_latin_capital_letter_i_with_macron,
  UCS2_latin_capital_letter_i_with_tilde,
  UCS2_latin_capital_letter_k_with_cedilla,
  UCS2_latin_capital_letter_l_with_cedilla,
  UCS2_latin_capital_letter_n_with_acute,
  UCS2_latin_capital_letter_r_with_cedilla,
  UCS2_latin_capital_letter_s_with_caron,
  UCS2_latin_capital_letter_t_with_stroke,
  UCS2_latin_capital_letter_z_with_caron,
  UCS2_soft_hyphen,
  UCS2_latin_small_letter_kra,
  UCS2_latin_capital_letter_eng,
  UCS2_latin_small_letter_d_with_stroke,
  UCS2_latin_small_letter_a_with_ogonek,
  UCS2_latin_small_letter_e_with_macron,
  UCS2_latin_small_letter_g_with_cedilla,
  UCS2_latin_small_letter_i_with_macron,
  UCS2_latin_small_letter_i_with_tilde,
  UCS2_latin_small_letter_k_with_cedilla,
  UCS2_latin_small_letter_l_with_cedilla,
  UCS2_latin_small_letter_n_with_acute,
  UCS2_latin_small_letter_r_with_cedilla,
  UCS2_latin_small_letter_s_with_caron,
  UCS2_latin_small_letter_t_with_stroke,
  UCS2_latin_small_letter_z_with_caron,
  UCS2_section_sign,
  UCS2_latin_small_letter_sharp_s,
  UCS2_latin_small_letter_eng,
  UCS2_latin_capital_letter_a_with_macron,
  UCS2_latin_capital_letter_a_with_acute,
  UCS2_latin_capital_letter_a_with_circumflex,
  UCS2_latin_capital_letter_a_with_tilde,
  UCS2_latin_capital_letter_a_with_diaeresis,
  UCS2_latin_capital_letter_a_with_ring_above,
  UCS2_latin_capital_ligature_ae,
  UCS2_latin_capital_letter_i_with_ogonek,
  UCS2_latin_capital_letter_c_with_caron,
  UCS2_latin_capital_letter_e_with_acute,
  UCS2_latin_capital_letter_e_with_ogonek,
  UCS2_latin_capital_letter_e_with_diaeresis,
  UCS2_latin_capital_letter_e_with_dot_above,
  UCS2_latin_capital_letter_i_with_acute,
  UCS2_latin_capital_letter_i_with_circumflex,
  UCS2_latin_capital_letter_i_with_diaeresis,
  UCS2_latin_capital_letter_d_with_stroke,
  UCS2_latin_capital_letter_n_with_cedilla,
  UCS2_latin_capital_letter_o_with_macron,
  UCS2_latin_capital_letter_o_with_acute,
  UCS2_latin_capital_letter_o_with_circumflex,
  UCS2_latin_capital_letter_o_with_tilde,
  UCS2_latin_capital_letter_o_with_diaeresis,
  UCS2_latin_capital_letter_u_with_tilde,
  UCS2_latin_capital_letter_o_with_stroke,
  UCS2_latin_capital_letter_u_with_ogonek,
  UCS2_latin_capital_letter_u_with_acute,
  UCS2_latin_capital_letter_u_with_circumflex,
  UCS2_latin_capital_letter_u_with_diaeresis,
  UCS2_latin_capital_letter_y_with_acute,
  UCS2_latin_capital_letter_thorn,
  UCS2_latin_capital_letter_u_with_macron,
  UCS2_latin_small_letter_a_with_macron,
  UCS2_latin_small_letter_a_with_acute,
  UCS2_latin_small_letter_a_with_circumflex,
  UCS2_latin_small_letter_a_with_tilde,
  UCS2_latin_small_letter_a_with_diaeresis,
  UCS2_latin_small_letter_a_with_ring_above,
  UCS2_latin_small_ligature_ae,
  UCS2_latin_small_letter_i_with_ogonek,
  UCS2_latin_small_letter_c_with_caron,
  UCS2_latin_small_letter_e_with_acute,
  UCS2_latin_small_letter_e_with_ogonek,
  UCS2_latin_small_letter_e_with_diaeresis,
  UCS2_latin_small_letter_e_with_dot_above,
  UCS2_latin_small_letter_i_with_acute,
  UCS2_latin_small_letter_i_with_circumflex,
  UCS2_latin_small_letter_i_with_diaeresis,
  UCS2_latin_small_letter_eth,
  UCS2_latin_small_letter_n_with_cedilla,
  UCS2_latin_small_letter_o_with_macron,
  UCS2_latin_small_letter_o_with_acute,
  UCS2_latin_small_letter_o_with_circumflex,
  UCS2_latin_small_letter_o_with_tilde,
  UCS2_latin_small_letter_o_with_diaeresis,
  UCS2_latin_small_letter_u_with_tilde,
  UCS2_latin_small_letter_o_with_stroke,
  UCS2_latin_small_letter_u_with_ogonek,
  UCS2_latin_small_letter_u_with_acute,
  UCS2_latin_small_letter_u_with_circumflex,
  UCS2_latin_small_letter_u_with_diaeresis,
  UCS2_latin_small_letter_y_with_acute,
  UCS2_latin_small_letter_thorn,
  UCS2_latin_small_letter_u_with_macron,
};

t_ucs2 ISOsamitoUCS2 (uchar c)
{
  return c >= 0xa0 ? ISOsami_to_UCS2_table[c-0xa0] : ASCIItoUCS2 (c);
}
